@@ -10,15 +10,12 @@ new_defaults = function(value = list()) {
   set = function(...) {
     dots = list(...)
     if (length(dots) == 0) return()
-    if (length(dots) == 1 && is.list(dots[[1]]))
+    if (is.null(names(dots)) && length(dots) == 1 && is.list(dots[[1]]))
       dots = dots[[1]]
     defaults <<- merge(dots)
     invisible(NULL)
   }
-  merge = function(values) {
-    defaults[names(values)] = values
-    defaults
-  }
+  merge = function(values) merge_list(defaults, values)
   restore = function() defaults <<- value
 
   list(get = get, set = set, merge = merge, restore = restore)
@@ -36,23 +33,23 @@ new_defaults = function(value = list()) {
 #' \url{http://yihui.name/knitr/options#chunk_options}
 #' @export
 #' @examples opts_chunk$get('prompt'); opts_chunk$get('fig.keep')
-opts_chunk = new_defaults(
-  list(eval = TRUE, echo = TRUE, results = 'markup', tidy = TRUE,
-       cache = FALSE, dependson = NULL, cache.path = 'cache/',
-       ref.label = NULL, child = NULL, engine = 'R',
-       prompt = FALSE, comment = '##', autodep = FALSE,
-       fig.keep = 'high', fig.show = 'asis', fig.align = 'default',
-       fig.path = 'figure/', fig.ext = NULL, dev = 'pdf', dpi = 72,
-       dev.args = NULL, fig.width = 7, fig.height = 7,
-       fig.cap = NULL, fig.scap = NULL, fig.lp = 'fig:', fig.pos = '',
-       out.width = NULL, out.height = NULL,
-       resize.width = NULL, resize.height = NULL,
-       external = TRUE, sanitize = FALSE,
-       highlight = TRUE, size = 'normalsize',
-       warning = TRUE, error = TRUE, message = TRUE,
-       background = '#F7F7F7', split = FALSE, include = TRUE,
-       interval = 1, aniopts = 'controls,loop')
-)
+opts_chunk = new_defaults(list(
+  eval = TRUE, echo = TRUE, results = 'markup', tidy = TRUE,
+  cache = FALSE, dependson = NULL, cache.path = 'cache/',
+  ref.label = NULL, child = NULL, engine = 'R',
+  prompt = FALSE, comment = '##', autodep = FALSE,
+  fig.keep = 'high', fig.show = 'asis', fig.align = 'default',
+  fig.path = 'figure/', fig.ext = NULL, dev = 'pdf', dpi = 72,
+  dev.args = NULL, fig.width = 7, fig.height = 7,
+  fig.cap = NULL, fig.scap = NULL, fig.lp = 'fig:', fig.pos = '',
+  out.width = NULL, out.height = NULL, out.extra = NULL,
+  resize.width = NULL, resize.height = NULL,
+  external = TRUE, sanitize = FALSE,
+  highlight = TRUE, size = 'normalsize',
+  warning = TRUE, error = TRUE, message = TRUE,
+  background = '#F7F7F7', split = FALSE, include = TRUE,
+  interval = 1, aniopts = 'controls,loop'
+))
 
 #' @rdname opts_chunk
 #' @export
@@ -82,17 +79,32 @@ opts_chunk_attr = (function() {
 #' \url{http://yihui.name/knitr/options#package_options}
 #' @export
 #' @examples opts_knit$get('verbose'); opts_knit$set(verbose = TRUE)  # change it
-opts_knit = new_defaults(
-  list(progress = TRUE, verbose = FALSE, out.format = NULL,
-       child.command = 'input', base.dir = NULL, base.url = NULL, child.path = '',
-       upload.fun = identity, eval.after = NULL, concordance = FALSE,
-       tangle = FALSE, child = FALSE, parent = FALSE,
-       cache.extra = NULL, aliases = NULL,
-       self.contained = TRUE, filter.chunk.end = TRUE,
-       header = c(highlight = '', tikz = '', framed = ''))
-)
+opts_knit = new_defaults(list(
+  progress = TRUE, verbose = FALSE, out.format = NULL,
+  child.command = 'input', base.dir = NULL, base.url = NULL, child.path = '',
+  upload.fun = identity, animation.fun = NULL,
+  eval.after = NULL, concordance = FALSE, sweave.penalty = 10,
+  tangle = FALSE, child = FALSE, parent = FALSE,
+  cache.extra = NULL, aliases = NULL, root.dir = NULL,
+  self.contained = TRUE, filter.chunk.end = TRUE, use.highlight = FALSE,
+  header = c(highlight = '', tikz = '', framed = '')
+))
 ## header should not be set by hand unless you know what you are doing
 
 ## tangle: whether I'm in tangle mode; child: whether I'm in child
 ## document mode; parent: whether I need to add parent preamble to the
 ## child output
+
+#' Template for creating reusable chunk options
+#'
+#' Creates a template binding a label to a set of chunk options. Every chunk
+#' that references the template label will have the specificed set of options
+#' applied to it.
+#' @export
+#' @examples opts_template$set(myfigures = list(fig.height = 4, fig.width = 4))
+#' # later you can reuse these chunk options by 'opts.label', e.g.
+#'
+#' # <<foo, opts.label='myfigures'>>=
+#'
+#' # the above is equivalent to <<foo, fig.height=4, fig.width=4>>=
+opts_template = new_defaults()
