@@ -24,6 +24,12 @@
 #' \command{convert} in ImageMagick is used to trim the white margins (call
 #' \command{convert input -trim output}).
 #'
+#' The function \code{hook_optipng} calls the program \command{optipng} to
+#' optimize PNG images. Note the chunk option \code{optipng} can be used to
+#' provide additional parameters to the program \command{optipng}, e.g.
+#' \code{optipng = '-o7'}. See \url{http://optipng.sourceforge.net/} for
+#' details.
+#'
 #' When the plots are not recordable via \code{\link[grDevices]{recordPlot}} and
 #' we save the plots to files manually via other functions (e.g. \pkg{rgl}
 #' plots), we can use the chunk hook \code{hook_plot_custom} to help write code
@@ -42,8 +48,7 @@ hook_rgl = function(before, options, envir) {
   par3d(windowRect = 100 + options$dpi * c(0, 0, options$fig.width, options$fig.height))
   Sys.sleep(.05) # need time to respond to window size change
 
-  fmt = opts_knit$get('out.format')
-  if (fmt %in% c('html', 'markdown', 'jekyll', 'rst')) options$dev = 'png'
+  if (out_format(c('html', 'markdown', 'jekyll', 'rst'))) options$dev = 'png'
 
   ## support 3 formats: eps, pdf and png (default)
   switch(options$dev,
@@ -68,8 +73,7 @@ hook_pdfcrop = function(before, options, envir) {
     warning('ImageMagick not installed or not in PATH')
     return()
   }
-  paths = fig_path(paste(if (fig.num == 1L) '' else seq_len(fig.num),
-                         ".", ext, sep = ""), options)
+  paths = all_figs(options, ext, fig.num)
 
   lapply(paths, function(x) {
     message('cropping ', x)
@@ -82,13 +86,34 @@ hook_pdfcrop = function(before, options, envir) {
 }
 #' @export
 #' @rdname chunk_hook
+hook_optipng = function(before, options, envir) {
+  if (before) return()
+  ext = tolower(options$fig.ext)
+  if (ext != 'png') {
+    warning('this hook only works with PNG at the moment'); return()
+  }
+  if (!nzchar(Sys.which('optipng'))) {
+    warning('cannot find optipng; please install and put it in PATH'); return()
+  }
+  paths = all_figs(options, ext)
+
+  lapply(paths, function(x) {
+    message('optimizing ', x)
+    x = shQuote(x)
+    cmd = paste('optipng', if (is.character(options$optipng)) options$optipng, x)
+    if (.Platform$OS.type == 'windows') cmd = paste(Sys.getenv("COMSPEC"), "/c", cmd)
+    system(cmd)
+  })
+  return()
+}
+#' @export
+#' @rdname chunk_hook
 hook_plot_custom = function(before, options, envir){
   if(before) return() # run hook after the chunk
 
-  ext = options$fig.ext
-  if(is.null(ext)) ext = dev2ext(options$dev)
+  ext = options$fig.ext %n% dev2ext(options$dev)
   name = fig_path()
-  fmt = opts_knit$get('out.format')
+  fmt = out_format()
   if (fmt %in% c('sweave', 'listings')) fmt = 'latex'
   hook = switch(fmt, latex = hook_plot_tex, html = hook_plot_html,
                 rst = hook_plot_rst, hook_plot_md)
